from django import template

register = template.Library()

@register.inclusion_tag("dj_chartjs/chart.html", takes_context=True)
def render_chart(context,values):
    chart = {
        "chart": values,
        "type_chart": context["type"],
        "legend_text": context.get("legend") if context.get("legend") else ""
    }        
        
    return chart


import json
import random
from abc import ABC, abstractmethod

'''
    Objects represent chartjs instances
'''
class ChartMixin(ABC):
    beginAtZero = True
    aspectRatio = False
    title = None
    legend = False
    type_chart = None

    @abstractmethod
    def generate_dataset(self):
        pass

    def generate_options(self):
        return {
            "responsive": True,
            "maintainAspectRatio": self.aspectRatio,
            "legend": {"display": self.legend},
            "title": {
                "fontSize": 14,
                "display": True if self.title is not None else False,
                "text": self.title if self.title is not None else ""
            }
        }


class BarChart(ChartMixin):

    type_chart = "bar"

    def generate_options(self):
        options = super().generate_options()
        options["scales"] = {
            "yAxes": [{
                "display": True,
                "ticks": {
                    "beginAtZero": self.beginAtZero,
                    "stepSize": 1
                }
            }],
        }
        return options

    def generate_dataset(self,labels,data,dataLabel=None):
        dataset = {
            "labels": list(labels),
            "datasets": [{
                "label": dataLabel if dataLabel is not None else "",
                "backgroundColor": ["#{:02x}{:02x}{:02x}".format(*map(lambda x: random.randint(0, 255), range(3))) for entry in labels],
                "data": list(data)
            }]
        }

        return {
            "type": self.type_chart,
            "data": json.dumps(dataset),
            "options": json.dumps(self.generate_options())
        }

class HorizontalBarChart(ChartMixin):

    type_chart = "horizontalBar"

    def generate_options(self):
        options = super().generate_options()
        options["scales"] = {
            "xAxes": [{
                "display": True,
                "ticks": {
                    "beginAtZero": self.beginAtZero,
                    "stepSize": 1
                }
            }],
        }
        return options

    def generate_dataset(self,labels,data,dataLabel=None):
        dataset = {
            "labels": list(labels),
            "datasets": [{
                "label": dataLabel if dataLabel is not None else "",
                "backgroundColor": ["#{:02x}{:02x}{:02x}".format(*map(lambda x: random.randint(0, 255), range(3))) for entry in labels],
                "data": list(data)
            }]
        }

        return {
            "type": self.type_chart,
            "data": json.dumps(dataset),
            "options": json.dumps(self.generate_options())
        }


class PieChart(ChartMixin):
    type_chart = "pie"

    def generate_options(self):
        context = super().generate_options()
        context["legend"] = {
            "position": "right"
        }
        return context

    def generate_dataset(self,labels,data,dataLabel=None):
        dataset = {
            "labels": list(labels),
            "datasets": [{
                "label": dataLabel if dataLabel is not None else "",
                "backgroundColor": ["#{:02x}{:02x}{:02x}".format(*map(lambda x: random.randint(0, 255), range(3))) for entry in labels],
                "data": list(data)
            }]
        }

        return {
            "type": self.type_chart,
            "data": json.dumps(dataset),
            "options": json.dumps(self.generate_options())
        }

class DoughnutChart(PieChart):
    type_chart = "doughnut"

class PolarAreaChart(PieChart):
    type_chart = "polarArea"

class LineChart(ChartMixin):
    type_chart = "line"

    def create_node(self,data,label,fill=False):
        return {
            "data": list(data),
            "label": label,
            "borderColor": "#{:02x}{:02x}{:02x}".format(*map(lambda x: random.randint(0, 255), range(3))),
            "fill": fill
        }

    def generate_options(self):
        options = super().generate_options()
        options["scales"] = {
            "yAxes": [{
                "display": True,
                "ticks": {
                    "beginAtZero": self.beginAtZero,
                    "stepSize": 1
                }
            }],
        }
        return options

    def generate_dataset(self,data,labels):
        dataset = { 
            "labels": labels,
            "datasets": data
        }

        return {
            "type": self.type_chart,
            "data": json.dumps(dataset),
            "options": json.dumps(self.generate_options())
        }
        

class GroupChart(ChartMixin):
    type_chart = "bar"

    def create_node(self,data,label):
        return {
            "label": label,
            "backgroundColor": "#{:02x}{:02x}{:02x}".format(*map(lambda x: random.randint(0, 255), range(3))),
            "data": list(data)
        }
        
    def generate_dataset(self,labels,data):
        dataset = { 
            "labels": list(labels),
            "datasets": list(data)
        }

        return {
            "type": self.type_chart,
            "data": json.dumps(dataset),
            "options": json.dumps(self.generate_options())
        }

class RadarChart(ChartMixin):
    
    type_chart = "radar"

    def create_node(self,label,data):
        color = self._get_color()
        return {
            "label": label,
            "fill": True,
            "backgroundColor": color,
            "borderColor": color,
            "pointBorderColor": "#fff",
            "pointBackgroundColor": color,
            "data": list(data)
        }
    
    def _get_color(self):
        return "rgba({},{},{},0.4)".format(
            *map(lambda x: random.randint(0, 255), range(5))
        )

    def generate_dataset(self,labels,data):
        dataset = { 
            "labels": list(labels),
            "datasets": list(data)
        }

        return {
            "type": self.type_chart,
            "data": json.dumps(dataset),
            "options": json.dumps(self.generate_options())
        }


from django.shortcuts import render
import json
from abc import ABC,abstractmethod
import random


class BaseChartView(ABC):

    type_chart = None
    title = None
    legend = False
    beginAtZero = False
    

    @abstractmethod
    def generate_values(self):
        pass

    def generate_options(self):
        options = {
            "legend": { "display": self.legend },
            "title": {
                "display": True if self.title is not None else False,
                "text": self.title
            },
        }
        return options
    
    @abstractmethod
    def generate_labels(self):
        pass

    def get_context_data(self,**kwargs):
        context = super().get_context_data(**kwargs)        
        context["chart"] = { "data": self._generate_data(), "options": self.generate_options()}
        context["type"] = self.type_chart
            
        return context


class BarChartView(BaseChartView):
    label = ""
    type_chart = "bar"

    def _generate_data(self):
        data = {
            "labels": self.generate_labels(),
            "datasets": self._generate_dataset(self.generate_values())
        }
        return json.dumps(data)

    def generate_options(self):
        options = super().generate_options()        
        options["scales"] = {
            "yAxes": [{
                "display": True,
                "ticks": {
                    "beginAtZero": True
                }
            }]
        }
           
        '''options["tooltips"] = {
            "callbacks": { 
                "label": json.dumps("function(tooltipItem,data){ var dataset = data.datasets[tooltipItem.datasetIndex]; return dataset; }")
            }
        }'''
            
        return json.dumps(options)
        
    
    def _generate_dataset(self,values):
        collection = []
        dataset = {
            "label": self.label,
            "backgroundColor": [self._get_color() for entry in self.generate_labels()],
            "data": values,
        }
        collection.append(dataset)
        return collection


    def _get_color(self):
        return "#{:02x}{:02x}{:02x}".format(
            *map(lambda x: random.randint(0, 255), range(3))
        )


class RadarChartView(BaseChartView):

    type_chart = "radar"

    def generate_labels(self):
        return []

    '''
        Create RadarNodes object and add to datasets list
    '''
    def generate_values(self):
        return []

    def create_node(self,label,data):
        color = self._get_color()
        return {
            "label": label,
            "fill": True,
            "backgroundColor": color,
            "borderColor": color,
            "pointBorderColor": "#fff",
            "pointBackgroundColor": color,
            "data": list(data)
        }
    
    def _get_color(self):
        return "rgba({},{},{},0.4)".format(
            *map(lambda x: random.randint(0, 255), range(5))
        )


    def _generate_data(self):        
        return json.dumps({
            "labels": self.generate_labels(),
            "datasets": self.generate_values() 
        })

    def generate_options(self):
        options = super().generate_options()
        return json.dumps(options)

    '''
        Generate random rgba colors
    '''
    def _get_color(self):
        return "rgba({},{},{},0.4)".format(
            *map(lambda x: random.randint(0, 255), range(5))
        )
    

class LineChartView(BaseChartView):

    type_chart = "line"

    """
        the params accept `label` string, and `data` list of values (numeric)
        fill is False to default
    """
    def create_node(self,label,data,fill=False):
        color = self._get_color()
        return {
            "data": list(data),
            "label": label,
            "borderColor": color,
            "backgroundColor": color,
            "fill": fill
        }

    def generate_options(self):
        options = super().generate_options()
        options["scales"] = {
            "yAxes": [{
                "display": True,
                "ticks": {
                    "beginAtZero": self.beginAtZero,
                    "stepSize": 1
                }
            }],
        }
        return json.dumps(options)

    def _get_color(self):
        return "rgba({},{},{},0.4)".format(
            *map(lambda x: random.randint(0, 255), range(5))
        )

    def _generate_data(self):        
        return json.dumps({
            "labels": self.generate_labels(),
            "datasets": self.generate_values() 
        })


class GroupChartView(BaseChartView):
    
    type_bar = "bar"

    def create_node(self,data,label):
        return {
            "label": label,
            "backgroundColor": "#{:02x}{:02x}{:02x}".format(*map(lambda x: random.randint(0, 255), range(3))),
            "data": list(data)
        }
        
    def _generate_data(self):        
        return json.dumps({
            "labels": self.generate_labels(),
            "datasets": self.generate_values() 
        })



class HorizontalBarChartView(BarChartView):
    type_chart = "horizontalBar"


class PolarAreaChartView(BarChartView):
    type_chart = "polarArea"


class PieChartView(BarChartView):
    type_chart = "pie"

    def generate_options(self):
        options = {
            "legend": { "display": self.legend, "position": "right" },
            "title": {
                "display": True if self.title is not None else False,
                "text": self.title
            },
        }      
            
        return json.dumps(options)
    
    def get_legend_text(self):
        return ""

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context["legend"] = self.get_legend_text()
        return context


class DoughnutChartView(PieChartView):
    type_chart = "doughnut"

    

